/** @file   ex039.c Demo-Programm fuer Datei-Attribute auf POSIX-Systmen.

    ex039 <Dateiname>
*/

/* Copyright (C) 2014-2017 - HS Schmalkalden. All rights reserved. */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

/** Hauptprogramm.
    @param  argc    Anzahl der Kommandozeilenargumente.
    @param  argv    Feld mit Kommandozeilenargumenten.
    @return 0 bei Erfolg, alle anderen Werte deuten auf Fehler hin.
*/
int
main(int argc, char *argv[])
{
    struct stat   stbuf;
    struct tm     timebuf;
    int           exc     =   EXIT_FAILURE;

    /* Es muss ein Argument angegeben sein: Dateiname */
    if (2 != argc) {
        /* !!! Falsche Argumente-Anzahl */
        goto finished;
    }

    /* Versuche, Datei-Attribute abzufragen */
    if (0 != stat(argv[1], &stbuf)) {
        /* !!! Abfrage der Dateiattribute fehlgeschlagen */
        goto finished;
    }

    /* Bis jetzt erfolgreich */
    exc = EXIT_SUCCESS;

    /* Allgemeine Informationen ueber Datei ausgeben */
    printf("Datei-Name:             %s\n",   argv[1]);
    printf("Groesse:                %ju\n",  (uintmax_t)(stbuf.st_size));

    /* Dateityp ermitteln und ausgeben */
    switch(stbuf.st_mode & S_IFMT) {
        case S_IFREG: {
            printf("Dateityp:               Regulaere Datei\n");
        } break;
        case S_IFDIR: {
            printf("Dateityp:               Directory\n");
        } break;
        case S_IFSOCK: {
            printf("Dateityp:               Socket\n");
        } break;
        case S_IFLNK: {
            printf("Dateityp:               Symbolischer Link\n");
        } break;
        case S_IFBLK: {
            printf("Dateityp:               Blockorientiertes Geraet\n");
        } break;
        case S_IFCHR: {
            printf("Dateityp:               Byteorientiertes Geraet\n");
        } break;
        case S_IFIFO: {
            printf("Dateityp:               FIFO (benannte Pipe)\n");
        } break;
        default: {
            printf("Dateityp:               Unbekannt\n");
            exc = EXIT_FAILURE;
        } break;
    }

    /* Eigentuemer ausgeben */
    printf("Eigentuemer-UID:        %ju\n",  (uintmax_t)(stbuf.st_uid));
    printf("Eigentuemer-GID:        %ju\n",  (uintmax_t)(stbuf.st_gid));
    
    printf(
        "Berechtigungen:         %c%c%c%c%c%c%c%c%c%c%c%c\n",
        ((0 != (stbuf.st_mode & S_ISUID)) ? 'u' : '-'),
        ((0 != (stbuf.st_mode & S_ISGID)) ? 'g' : '-'),
        ((0 != (stbuf.st_mode & S_ISVTX)) ? 's' : '-'),
        ((0 != (stbuf.st_mode & S_IRUSR)) ? 'r' : '-'),
        ((0 != (stbuf.st_mode & S_IWUSR)) ? 'w' : '-'),
        ((0 != (stbuf.st_mode & S_IXUSR)) ? 'x' : '-'),
        ((0 != (stbuf.st_mode & S_IRGRP)) ? 'r' : '-'),
        ((0 != (stbuf.st_mode & S_IWGRP)) ? 'w' : '-'),
        ((0 != (stbuf.st_mode & S_IXGRP)) ? 'x' : '-'),
        ((0 != (stbuf.st_mode & S_IROTH)) ? 'r' : '-'),
        ((0 != (stbuf.st_mode & S_IWOTH)) ? 'w' : '-'),
        ((0 != (stbuf.st_mode & S_IXOTH)) ? 'x' : '-')
    );

    /* Weitere Informationen ausgeben */
    printf("Link-Anzahl:            %ju\n",  (uintmax_t)(stbuf.st_nlink));
    printf("Device-ID:              %ju\n",  (uintmax_t)(stbuf.st_dev));
    printf("Rel. Device-ID:         %ju\n",  (uintmax_t)(stbuf.st_rdev));
    printf("Inode-Nummer:           %ju\n",  (uintmax_t)(stbuf.st_ino));
    printf("Blockgroesse:           %ju\n",  (uintmax_t)(stbuf.st_blksize));
    printf("Block-Anzahl:           %ju\n",  (uintmax_t)(stbuf.st_blocks));

    /* Zeitpunkt ausgeben, wann Datei angelegt wurde */
    if (NULL != localtime_r(&(stbuf.st_ctime), &timebuf)) {
        printf(
            "Erzeugungszeitpunkt:    %04d-%02d-%02d %02d:%02d:%02d\n",
            (1900 + timebuf.tm_year),
            (1 + timebuf.tm_mon),
            timebuf.tm_mday,
            timebuf.tm_hour,
            timebuf.tm_min,
            timebuf.tm_sec
        );
    }
    else {
        exc = EXIT_FAILURE;
    }

    /* Zeitpunkt der letzten Aenderung ausgeben */
    if (NULL != localtime_r(&(stbuf.st_mtime), &timebuf)) {
        printf("Modifikationszeitpunkt: %04d-%02d-%02d %02d:%02d:%02d\n",
            (1900 + timebuf.tm_year),
            (1 + timebuf.tm_mon),
            timebuf.tm_mday,
            timebuf.tm_hour,
            timebuf.tm_min,
            timebuf.tm_sec
        );
    }
    else {
        exc = EXIT_FAILURE;
    }

    /* Zeitpunkt des letzten Zugriffes auf Datei ausgeben */
    if (NULL != localtime_r(&(stbuf.st_atime), &timebuf)) {
        printf("Zugriffszeitpunkt:      %04d-%02d-%02d %02d:%02d:%02d\n",
            (1900 + timebuf.tm_year),
            (1 + timebuf.tm_mon),
            timebuf.tm_mday,
            timebuf.tm_hour,
            timebuf.tm_min,
            timebuf.tm_sec
        );
    }
    else {
        exc = EXIT_FAILURE;
    }

finished:
    exit(exc); return exc;
}

/* vim: set ai sw=4 ts=4 expandtab : */
