/** @file	ex083.c	Demonstation von calloc() und free().
    Das Programm erhaelt keine Argumente uebergeben.
*/

#include "he-conf.h"
#include <stdio.h>
#include <stdlib.h>
#if HAVE_LIMITS_H
#include <limits.h>
#endif
#if HAVE_STDINT_H
#include <stdint.h>
#endif

#include "he-read.h"


/** Status-Code des Programmes, wird von main-Funktion zurueckgegeben.
*/
static int excode = EXIT_FAILURE;


/** Gleitkommazahlen in Puffer einlesen.
    @param	buffer  Puffer, der mit Werten gefuellt werden soll.
    @param	anz     Groesse des Puffers (Anzahl der Werte).
    @return 1 bei Erfolg, 0 bei Fehler, z.B. Abbruch oder ungueltige Eingabe.
*/
static
int
werte_einlesen(double *buffer, size_t anz)
{
    int		 back	= 1;    /* Ergebnis der Funktion */
    size_t	 i;             /* Index aktuell bearbeiteter Wert */

    for (i = 0; ((i < anz) && (back == 1)); i++) {
        if (0 == hsm_et_read_double("Wert: ", buffer++)) {
            back = 0;
        }
    }
    return back;
}


/** Mittelwert mehrerer Werte berechnen.
    @param	buffer  Adresse des Puffers, in dem die Werte stehen.
    @param	anz     Anzahl der Werte im Puffer, muss positiv sein.
*/
static
void
mittelwert_berechnen(double *buffer, size_t anz)
{
    double	summe = 0.0;    /* Summe der Elemente */
    size_t	i;              /* Index aktuell bearbeitetes Element */

    for (i = 0; i < anz; i++) {
        summe += *(buffer++);
    }
    summe = summe / (double)anz;
    printf("Mittelwert: %g\n", summe);
}


/** Puffer fuer mehrere Werte dynamisch anfordern, Werte einlesen,
    Mittelwert berechnen und ausgeben.
    @param	anz	Anzahl der Werte, muss positiv sein.
*/
static
void
durchschnitt_mehrerer_werte(int anz)
{
    double		*buffer	= NULL;    /* Dynamisch allozierter Puffer fuer Werte */
    size_t	 	 sz_anz;           /* Anzahl Werte im Datentyp size_t */

    sz_anz = (size_t)anz;
    if ((SIZE_MAX / sizeof(double)) >= sz_anz) {
        buffer = calloc(sz_anz, sizeof(double));
        if (NULL != buffer) {
            /*
                Der von calloc() erhaltene Zeiger ist ungleich NULL,
                wir koennen den Speicher verwenden.
            */
            if (0 != werte_einlesen(buffer, sz_anz)) {
                mittelwert_berechnen(buffer, sz_anz);
                excode = EXIT_SUCCESS;
            }
            /*  Speicher nach Benutzung wieder freigeben
            */
            free(buffer);
        }
        else {
            /*
                Der von calloc() erhaltene Zeiger ist NULL,
                die Speicheranforderung ist fehlgeschlagen!
            */
            printf("FEHLER: Speicher nicht erhalten!\n");
        }
    }
    else {
        printf("FEHLER: Numerischer Ueberlauf, die Anzahl ist zu gross!\n");
    }
}


/** Hauptprogramm.
    @return	EXIT_SUCCESS (0) bei Erfolg, EXIT_FAILURE (ungleich 0) bei Fehler.
*/
int
main(void)
{
    int		anzahl	= 0;	/* Anzahl der Werte */

    printf("Mittelwertberechnung\n");
    if (0 != hsm_et_read_int("Anzahl der Werte: ", &anzahl)) {
        if (0 < anzahl) {
            durchschnitt_mehrerer_werte(anzahl);
        }
        else {
            printf("FEHLER: Die Anzahl muss positiv sein!\n");
        }
    }
    return excode;
}

/* vim: set ai sw=4 ts=4 : */
