/** @file   he-long.c   Funktionen fuer mathematische Operationen
    mit long-Werten, Erkennung von Ueberlauf, Vorbeugung vor Division
    durch 0.
*/

/* Copyright (C) 2014 - HS Schmalkalden. All rights reserved. */

#include "he-conf.h"

#include <stdio.h>
#if HAVE_STDLIB_H
#include <stdlib.h>
#endif
#if HAVE_STDINT_H
#include <stdint.h>
#endif
#if HAVE_LIMITS_H
#include <limits.h>
#endif

#include "he-long.h"
#include "he-error.h"
#include "he-numco.h"

/*  Addition.
    a	Linker Operand.
    b	Rechter Operand.
    ec	Zeiger auf Fehlercode-Variable, darf NULL sein.
    Rueckgabe:	Ergebnis.
*/
long
hsm_et_long_add(long a, long b, int *ec)
{
  if (NULL != ec) {
    if ((0L < a) && (0L < b)) {
      if ((LONG_MAX - a) < b) {
        hsm_et_error_set(ec, HE_ERROR_MATH_OVERFLOW);
      }
    } else {
      if ((0L > a) && (0L > b)) {
        if (LONG_MIN == a) {
          hsm_et_error_set(ec, HE_ERROR_MATH_OVERFLOW);
        } else {
          if ((LONG_MIN - a) > b) {
            hsm_et_error_set(ec, HE_ERROR_MATH_OVERFLOW);
          }
        }
      }
    }
  }
  return (a + b);
}



/*	Subtraktion.
    a	Linker Operand.
    b	Rechter Operand.
    ec	Zeiger auf Fehlercode-Variable, darf NULL sein.
    Rueckgabe	Ergebnis.
*/
long
hsm_et_long_sub(long a, long b, int *ec)
{
  if (NULL != ec) {
    if ((0L < a) && (0L > b)) {
      if ((LONG_MAX + b) < a) {
        hsm_et_error_set(ec, HE_ERROR_MATH_OVERFLOW);
      }
    } else {
      if ((0L > a) && (0L < b)) {
        if ((LONG_MIN + b) > a) {
          hsm_et_error_set(ec, HE_ERROR_MATH_OVERFLOW);
        }
      }
    }
  }
  return (a - b);
}



/*  Multiplikation.
    a	Linker Operand.
    b	Rechter Operand.
    ec	Zeiger auf Fehlercode-Variable, darf NULL sein.
    Rueckgabe:	Ergebnis.
*/
long
hsm_et_long_mul(long a, long b, int *ec)
{
  if ((0L != a) && (0L != b)) {
    if (LONG_MIN == a) {
      if (1L != b) {
        hsm_et_error_set(ec, HE_ERROR_MATH_OVERFLOW);
      }
    } else {
      if (LONG_MIN == b) {
        if (1L != a) {
          hsm_et_error_set(ec, HE_ERROR_MATH_OVERFLOW);
        }
      }
      if (0L < a) {
        if (0L < b) {
          if ((LONG_MAX / a) < b) {
            hsm_et_error_set(ec, HE_ERROR_MATH_OVERFLOW);
          }
        } else {
          if ((LONG_MIN / a) > b) {
            hsm_et_error_set(ec, HE_ERROR_MATH_OVERFLOW);
          }
        }
      } else {
        if (0L < b) {
          if ((LONG_MIN / b) > a) {
            hsm_et_error_set(ec, HE_ERROR_MATH_OVERFLOW);
          }
        } else {
          if ((LONG_MAX / (0L - a)) < (0L - b)) {
            hsm_et_error_set(ec, HE_ERROR_MATH_OVERFLOW);
          }
        }
      }
    }
  }
  return (a * b);
}



/*  Division
    a	Zaehler.
    b	Nenner.
   	ec	Zeiger auf Fehlercode-Variable, darf NULL sein.
    Rueckgabe:	Ergebnis.
*/
long
hsm_et_long_div(long a, long b, int *ec)
{
  long     result   =   0L;
  if (0L != b) {
    if (LONG_MIN == a) {
      if (-1L == b) {
        result = LONG_MAX;
        hsm_et_error_set(ec, HE_ERROR_MATH_DIVZERO);
      } else {
        result = a / b;
      }
    } else {
      result = a / b;
    }
  } else {
    if (0L <= a) {
      result = LONG_MAX;
    } else {
      result = LONG_MIN;
    }
    hsm_et_error_set(ec, HE_ERROR_MATH_DIVZERO);
  }
  return result;
}



/*  Betrag bilden.
    a	Linker Operand.
    ec	Zeiger auf Fehlercode-Variable, darf NULL sein.
    Rueckgabe:	Ergebnis.
*/
long
hsm_et_long_abs(long a, int *ec)
{
  long	result	=	0L;
  if (LONG_MIN == a) {
    result = LONG_MAX;
	hsm_et_error_set(ec, HE_ERROR_MATH_OVERFLOW);
  } else {
    result = ((0L <= a) ? a : (0L - a));
  }
  return result;
}



/*  Quadrieren.
    a	Linker Operand.
    ec	Zeiger auf Fehlercode-Variable, darf NULL sein.
    Rueckgabe:	Ergebnis.
*/
long
hsm_et_long_sqr(long a, int *ec)
{
  return (hsm_et_long_mul(a, a, ec));
}


/* vim: set ai sw=4 ts=4 expandtab : */

