/** @file   ex034.c Demonstrationsprogramm zur Ressourcenfreigabe.

	ex034 Quelldatei Zieldatei
*/

/* Copyright (C) 2014-2017 - HS Schmalkalden. All rights reserved. */

#include "he-conf.h"

#if HE_ON_WINDOWS
#define _CRT_SECURE_NO_WARNINGS 1
#endif

#include <stdio.h>

#if HAVE_STDLIB_H
#include <stdlib.h>
#endif
#if HAVE_UNISTD_H
#include <unistd.h>
#endif
#if HAVE_PROCESS_H
#include <process.h>
#endif
#if HAVE_STDINT_H
#include <stdint.h>
#endif
#if HAVE_LIMITS_H
#include <limits.h>
#endif

#include "he-file.h"

/** Hauptprogramm.
	@param  argc    Anzahl der Kommandozeilen-Argumente.
	@param  argv    Kommandozeilen-Argumente.
	@return 0 bei Erfolg, alle anderen Werte weisen auf Fehler in.
*/
int
main(int argc, char *argv[])
{
	FILE      *infile     =   NULL;           /* Eingabedatei */
	FILE      *outfile    =   NULL;           /* Ausgabedatei */
	void      *puffer     =   NULL;           /* Puffer fuer Dateiinhalt */
	uintmax_t  filesize;                      /* Dateigroesse */
	size_t     rdbytes;                       /* Anzahl gelesene Bytes */
	size_t     wrbytes;                       /* Anzahl geschriebene Bytes */
	int        exc        =   EXIT_FAILURE;   /* Exit-Status-Code */

	/* Anzahl an Kommandozeilenargumenten testen */
	if (3 != argc) {
		/* Falsche Anzahl an Kommandozeilenargumenten */
		fputs("FEHLER: Falsche Anzahl an Kommandozeilen-Argumenten!\n", stderr);
		fputs("ex032 Quelldatei Zieldatei\n", stderr);
		fflush(stderr);
		goto finished;
	}

	/* Eingabedatei oeffnen */
	infile = fopen(argv[1], "rb");
	if (NULL == infile) {
		/* Quelldatei kann nicht gelesen werden */
		fputs("FEHLER: Quelldatei \"", stderr);
		fputs(argv[1], stderr);
		fputs("\" kann nicht gelesen werden!\n", stderr);
		fflush(stderr);
		goto finished;
	}

	/* Versuchen, die Groesse der Quelldatei zu bestimmen */
	if (0 == hsm_et_file_pointer_size(&filesize, infile, argv[1], NULL)) {
		/* Groesse der Quelldatei kann nicht bestimmt werden */
		fputs("FEHLER: Groesse der Quelldatei \"", stderr);
		fputs(argv[1], stderr);
		fputs("\" kann nicht bestimmt werden!\n", stderr);
		fflush(stderr);
		goto close_input_file;
	}

	/* Groessentest, Wert muss positiv sein */
	if (0 == filesize) {
		/* Quelldatei hat Groesse 0 */
		fputs("FEHLER: Quelldatei \"", stderr);
		fputs(argv[1], stderr);
		fputs("\" hat Groesse 0!\n", stderr);
		fflush(stderr);
		goto close_input_file;
	}

	/* Groessentest, Wert muss als size_t abbildbar sein */
	if ((uintmax_t)SIZE_MAX < filesize) {
		/* Dateigroesse zu gross fuer size_t */
		fputs("FEHLER: Quelldatei \"", stderr);
		fputs(argv[1], stderr);
		fputs("\" ist zu gross!\n", stderr);
		fflush(stderr);
		goto close_input_file;
	}

	/* Speicher dynamisch allozieren, Test */
	puffer = calloc(1, filesize);
	if (NULL == puffer) {
		/* Speicherallokation fehlgeschlagen */
		fputs("FEHLER: Speicherallokation fehlgeschlagen!\n", stderr);
		fflush(stderr);
		goto close_input_file;
	}

	/* Einlesen, Test ob vollstaendig eingelesen wurde. */
	rdbytes = fread(puffer, 1, filesize, infile);
	if ((uintmax_t)rdbytes < filesize) {
		/* Nicht alle Bytes eingelesen */
		fputs("FEHLER: Zu wenige Bytes aus Datei \"", stderr);
		fputs(argv[1], stderr);
		fputs("\" gelesen!\n", stderr);
		fflush(stderr);
		goto release_memory;
	}

	/* Ausgabedatei oeffnen */
	outfile = fopen(argv[2], "wb");
	if (NULL == outfile) {
		/* Zieldatei nicht zum Schreiben geoeffnet */
		fputs("FEHLER: Zieldatei \"", stderr);
		fputs(argv[2], stderr);
		fputs("\" nicht zum Schreiben geoeffnet!\n", stderr);
		fflush(stderr);
		goto release_memory;
	}

	/* Dateiinhalt schreiben */
	wrbytes = fwrite(puffer, 1, rdbytes, outfile);
	if ((uintmax_t)wrbytes < rdbytes) {
		/* Fehler beim Schreiben der Daten */
		fputs("FEHLER: Zu wenige Bytes in Datei \"", stderr);
		fputs(argv[2], stderr);
		fputs("\" geschrieben!\n", stderr);
		fflush(stderr);
		goto close_output_file;
	}

	/* Hier wurde alles erfolgreich abgearbeitet. */
	exc = EXIT_SUCCESS;

close_output_file:
	if (0 != fclose(outfile)) {
		/* Fehler beim Schliessen der Zieldatei */
		fputs("FEHLER: Zieldatei \"", stderr);
		fputs(argv[2], stderr);
		fputs("\" nicht korrekt geschlossen!\n", stderr);
		fflush(stderr);
		exc = EXIT_FAILURE;
	}

release_memory:
	free(puffer);

close_input_file:
	fclose(infile);

finished:
	exit(exc); return exc;
}
