/** @file   he-ulong.c  Funktionen fuer Berechnungen mit unsigned-long-Werten.
	Erkennung von Ueberlaeufen, Vermeidung von Division durch 0.
*/

/* Copyright (C) 2014-2017 - HS Schmalkalden. All rights reserved. */

#include "he-conf.h"

#if HAVE_STDLIB_H
#include <stdlib.h>
#endif
#if HAVE_STDINT_H
#include <stdint.h>
#endif
#if HAVE_LIMITS_H
#include <limits.h>
#endif

#include "he-ulong.h"
#include "he-error.h"
#include "he-numco.h"


/*  Addition.
		a	Linker Operand.
		b	Rechter Operand.
		ec	Zeiger auf Fehlercode-Variable, darf NULL sein.
		Rueckgabe: Ergebnis.
*/
unsigned long
hsm_et_ulong_add(unsigned long a, unsigned long b, int *ec)
{
	if (NULL != ec) {
		if ((ULONG_MAX - a) < b) {
			hsm_et_error_set(ec, HE_ERROR_MATH_OVERFLOW);
		}
	}
	return (a + b);
}



/*  Subtraktion.
		a	Linker Operand.
		b	Rechter Operand.
		ec	Zeiger auf Fehlercode-Variable, darf NULL sein.
		Rueckgabe: Ergebnis.
*/
unsigned long
hsm_et_ulong_sub(unsigned long a, unsigned long b, int *ec)
{
	if (NULL != ec) {
		if (b > a) {
			hsm_et_error_set(ec, HE_ERROR_MATH_OVERFLOW);
		}
	}
	return (a - b);
}



/*  Multiplikation.
		a	Linker Operand.
		b	Rechter Operand.
		ec	Zeiger auf Fehlercode-Variable, darf NULL sein.
		Rueckgabe: Ergebnis.
*/
unsigned long
hsm_et_ulong_mul(unsigned long a, unsigned long b, int *ec)
{
	if (NULL != ec) {
		if ((0UL != a) && (0UL != b)) {
			if ((ULONG_MAX / a) < b) {
				hsm_et_error_set(ec, HE_ERROR_MATH_OVERFLOW);
			}
		}
	}
	return (a * b);
}



/*  Division.
		a	Zaehler.
		b	Nenner.
		ec	Zeiger auf Fehlercode-Variable, darf NULL sein.
		Rueckgabe: Ergebnis.
*/
unsigned long
hsm_et_ulong_div(unsigned long a, unsigned long b, int *ec)
{
	unsigned long result  =   0UL;
	if (0UL != b) {
		result = a / b;
	}
	else {
		result = ULONG_MAX;
		hsm_et_error_set(ec, HE_ERROR_MATH_DIVZERO);
	}
	return result;
}



/*  Quadrieren.
		a	Linker Operand.
		ec	Zeiger auf Fehlercode-Variable, darf NULL sein.
		Rueckgabe: Ergebnis.
*/
unsigned long
hsm_et_ulong_sqr(unsigned long a, int *ec)
{
	return (hsm_et_ulong_mul(a, a, ec));
}



/* vim: set ai sw=4 ts=4 : */

