
#define	_CRT_SECURE_NO_WARNINGS	1
#define	_USE_MATH_DEFINES	1

#include <stdio.h>
#include <math.h>

#include "he-read.h"



/** Temperaturspannung, 26mV.
*/
#define UT  26.0e-3



/** Funktion, deren Nullstelle gesucht wird.
    @param  uges    Gesamtspannung.
    @param  r       Widerstandswert.
    @param  is      Saettigungsstrom.
    @param  i       Aktuell zu betrachtender Strom (x-Wert).
    @return Aktueller Funktionswert.
*/
double f(
  double    uges,
  double    r,
  double    is,
  double    i
)
{
  return (r*i + UT*log(1.0 + i/is) - uges);
}



/** Erste Ableitung der Funktion, deren Nullstelle gesucht wird.
    @param  uges    Gesamtspannung.
    @param  r       Widerstandswert.
    @param  is      Saettigungsstrom.
    @param  i       Aktuell zu betrachtender Strom (x-Wert).
    @return Aktueller Wert der ersten Ableitung.
*/
double dfdx(
  double    uges,
  double    r,
  double    is,
  double    i
)
{
  return (r + UT/(i+is));
}



/** Berechnung des Stromes durch Halbierungsverfahren.
    @param  uges    Gesamtspannung.
    @param  r       Widerstandswert.
    @param  is      Saettigungsstrom der Diode.
    @param  eps     Gewuenschte Genauigkeit.
*/
static
void
halbierung(
  double    uges,
  double    r,
  double    is,
  double    eps
)
{
  double        a;      /* Linke Intervallgrenze */
  double        b;      /* Rechte Intervallgrenze */
  double        x;      /* Aktueller Strom */
  double        y;      /* Funktionswert */
  unsigned long passes; /* Anzahl Durchlaeufe */

  passes = 0UL;

  /* Strom ist groesser als 0, daher ist 0 gute linke Intervallgrenze */
  a = 0.0;

  /* Strom ohne Diode waere hoeher, somit geeignete rechte Intervallgrenze */
  b = uges / r;

  do {

    /* Aktuelle Durchlauf-Nummer erhoehen */
    passes++;

    /* Neuen x-Wert ermitteln */
    x = (a + b) / 2.0;

    /* Funktionswert fuer x-Wert berechnen */
    y = f(uges, r, is, x);

    /* Eine der Intervallgrenzen neu setzen */
    if (0.0 >= y) {
      a = x;
    } else {
      b = x;
    }

    /* Ausgabe */
    printf("H Pass %4lu    I=%lg   Delta=%lg\n", passes, x, y);

  } while(((fabs(a-b) > eps) || (fabs(y) > eps)) && (2000UL > passes));
  printf("\n");
}



/** Berechnung des Stromes mit Regula falsi.
    @param  uges    Gesamtspannung.
    @param  r       Widerstandswert.
    @param  is      Saettigungsstrom der Diode.
    @param  eps     Gewuenschte Genauigkeit.
*/
static
void
regula_falsi(
  double    uges,
  double    r,
  double    is,
  double    eps
)
{
  double        a;      /* Linke Intervallgrenze */
  double        fa;     /* Funktionswert von a */
  double        b;      /* Rechte Intervallgrenze */
  double        fb;     /* Funktionswert von b */
  double        x;      /* Aktueller Strom */
  double        y;      /* Funktionswert */
  unsigned long passes; /* Anzahl Durchlaeufe */

  passes = 0UL;

  /* Strom ist groesser als 0, daher ist 0 gute linke Intervallgrenze */
  a = 0.0;

  /* Strom ohne Diode waere hoeher, somit geeignete rechte Intervallgrenze */
  b = uges / r;

  fa = f(uges, r, is, a);
  fb = f(uges, r, is, b);
  do {
    
    /* Aktuelle Durchlauf-Nummer erhoehen */
    passes++;

    /* Neuen x-Wert ermitteln */
    x = (a * fb - b * fa) / (fb - fa);

    /* Funktionswert fuer x-Wert berechnen */
    y = f(uges, r, is, x);

    /* Eine der Intervallgrenzen neu setzen, Funktionswert auch speichern  */
    if (0.0 >= y) {
      a = x; fa = y;
    } else {
      b = x; fb = y;
    }

    /* Ausgabe */
    printf("R Pass %4lu    I=%lg   Delta=%lg\n", passes, x, y);

  } while(((fabs(a-b) > eps) || (fabs(y) > eps) )&& (2000UL > passes));
  printf("\n");
}



/** Berechnung des Stromes mit Newton-Verfahren.
    @param  uges    Gesamtspannung.
    @param  r       Widerstandswert.
    @param  is      Saettigungsstrom der Diode.
    @param  eps     Gewuenschte Genauigkeit.
*/
static
void
newton(
  double    uges,
  double    r,
  double    is,
  double    eps
)
{
  double        x;      /* Aktueller Strom-Wert */
  double        y;      /* Funktionswert */
  double        ys;     /* Ableitung */
  double        xold;   /* Vorheriger x-Wert */
  unsigned long passes; /* Anzahl Durchlaeufe */

  passes = 0UL;

  /* Anfangswert ist Strom, der ohne Diode fliessen wuerde */
  x = uges / r;

  do {

    /* Aktuelle Durchlauf-Nummer erhoehen */
    passes++;

    /* Funktionswert und erste Ableitung fuer aktuellen x-Wert bestimmen */
    y = f(uges, r, is, x);
    ys = dfdx(uges, r, is, x);

    /* x-Wert aufheben fuer Test, wie gross die Aenderung war */
    xold = x;

    /* Neuen x-Wert berechnen */
    x = x - y / ys;

    /* Ausgabe */
    printf("N Pass %4lu    I=%lg   Delta=%lg\n", passes, x, y);

  } while(((fabs(x - xold) > eps) || (fabs(y) > eps)) && (2000UL > passes));
  printf("\n");
}



/** Hauptprogramm.
    Es werden keine Kommandozeilenargumente verarbeitet, die Eingaben
    werden abgefragt.
    @return 0.
*/
int main(void)
{
  double    uges;   /* Gesamtspannung */
  double    r;      /* Widerstandswert */
  double    is;     /* Saettigungsstrom */
  double    eps;    /* Genauigkeit */

  if (0 != hsm_et_read_double("Gesamtspannung in V: ", &uges)) {
    if (0 != hsm_et_read_double("Widerstand in Ohm: ", &r)) {
      if (0 != hsm_et_read_double("Saettigungsstrom in Ampere: ", &is)) {
        if (0 != hsm_et_read_double("Genauigkeit: ", &eps)) {
          halbierung(uges, r, is, eps);
          regula_falsi(uges, r, is, eps);
          newton(uges, r, is, eps);
        }
      }
    }
  }
  return 0;
}

/* vim: set ai sw=4 ts=4 expandtab : */

